%
% bf_FFTquin2D_analysis.m, 04 Mar 2009 15:10 (+0100)
%
% Author: D. Van De Ville
%         Biomedical Imaging Group <http://bigwww.epfl.ch/>
%         Ecole Polytechnique Federale de Lausanne (EPFL)
%
% You are free to use this software for research purposes.  Redistribution
% without the consent of the Biomedical Imaging Group is not permitted.  This
% software is provided "AS IS" and any and all warranties, either express or
% implied, are hereby disclaimed.  In no event shall the EPFL or the developers
% be liable for any direct, indirect, incidental, special, exemplary,
% consequential, or other damages arising in any way out of the use of this
% software.
% 
% If you use this software in your research please consider citing the following
% reference(s).
% 
% [1] P. D. Tafti, D. Van De Ville, and M. Unser, "Invariances, Laplacian-like
%     wavelet bases, and the whitening of fractal processes", IEEE Transactions
%     on Image Processing, v. 18, no. 4, Apr. 2009.
% 
% [2] D. Van De Ville, T. Blu, M. Unser, "Isotropic Polyharmonic B-Splines:
%     Scaling Functions and Wavelets", IEEE Transactions on Image Processing,
%     vol.  14, no. 11, pp. 1798-1813, Nov. 2005.
%


function y2=FFTquin2D_analysis(x,J,FA);

% FFTQUIN2D_ANALYSIS Perform 2D quincunx wavelet analysis
% 	y2 = FFTquin2D_analysis(x,J,FA)
%
% 	Input:
%       x  = input data
%       J  = number of iterations
%       FA = analysis filters
%
%       Output:
%       y2 = result (folded)
%
% Dimitri Van De Ville
%      Biomedical Imaging Group, BIO-E/STI
%      Swiss Federal Institute of Technology Lausanne
%      CH-1015 Lausanne EPFL, Switzerland

% Dimensions of data
[m,n]=size(x); 

% Dimensions of remaining folded lowpass subband
cm=m; cn=n; 

% Check dimensions of input data
for iter=1:2,
  tmp=size(x,iter)/2^floor((J+2-iter)/2);
  if round(tmp)~=tmp,
    disp(' ')
    disp('The size of the input signal must be a sufficient power of two!')
    disp(' ')
    y2=[];
    return;
  end;
end;

% Analysis filters:
%  H1,   G1   : filters for iterations with mod(j,2)=1
%  H1D,  G1D  : filters for iterations with mod(j,2)=0
% (premultiply analysis filters by the downsampling factor; i.e., det(D)=2)
H1=FA(:,:,1)/2;
G1=FA(:,:,2)/2;
H1D=FA(:,:,3)/2;
G1D=FA(:,:,4)/2;

% Compute the FFT of the input data
X=fftn(x); clear x;

% Initialize cube for folded wavelet coefficients
y2=zeros(cm,cn);

for j=1:J,

	mj=mod(j,2);

	% Select filter
	switch mj,
	  case 1,	
		%H=H1; Original
		%G=G1; Original
		H=conj(H1);
		G=conj(G1);
	  case 0,
		%H=H1D; Original
		%G=G1D; Original
		H=conj(H1D);
		G=conj(G1D);
	end;
	
	% Filtering
	Y1=H.*X;
	Y2=G.*X;

	% Downsampling & upsampling
	switch mj,
	  case 1,
		Y1=Y1 + fftshift(Y1); 
		Y2=Y2 + fftshift(Y2);
		y2(1:cm,cn/2+1:cn)=fold2D(ifftn(Y2),mj,m,n);
		cn=cn/2;
	  case 0,
		m2=m/2;n2=n/2;
		Y1=Y1+fftshift(Y1,1);
		Y2=Y2+fftshift(Y2,1);	
		Y1=Y1(1:m2,1:n2);
		Y2=Y2(1:m2,1:n2);
		y2(cm/2+1:cm,1:cn)=fold2D(ifftn(Y2),mj,m,n);
		cm=cm/2;

		% Preparing filters for next three iterations
		lm=1:2:m; ln=1:2:n; 
		H1=H1(lm,ln);
		G1=G1(lm,ln);
        	H1D=H1D(lm,ln);
        	G1D=G1D(lm,ln);
		m=m2; n=n2; 
	end;

	X=Y1;
end;

% Insert folded lowpass subband
y2(1:cm,1:cn)=fold2D(ifftn(Y1),mj,m,n);
return;


% ----------------------------------------------------------------------
% Fold 2D subband
% ----------------------------------------------------------------------
function f=fold2D(u,j,m,n)
switch j,
  case 0,
    f=u;
  case 1,
    f=u(:,1:2:n)+u(:,2:2:n);
end;
